-- Bind necessary Android classes
ActivityManager = luajava.bindClass("android.app.ActivityManager")
Toast = luajava.bindClass("android.widget.Toast")
Handler = luajava.bindClass("android.os.Handler")
Looper = luajava.bindClass("android.os.Looper")
Runnable = luajava.bindClass("java.lang.Runnable")

-- Get context
ctx = activity or this

-- Function to safely show toast and exit
function showToastAndExit(msg)
  local handler
  local ok, err = pcall(function()
    handler = Handler(Looper.getMainLooper()) -- Proper static call
    handler:post(luajava.createProxy("java.lang.Runnable", {
      run = function()
        Toast.makeText(ctx, msg, Toast.LENGTH_LONG):show()
        handler:postDelayed(luajava.createProxy("java.lang.Runnable", {
          run = function()
            os.exit()
          end
        }), 2500)
      end
    }))
  end)

  if not ok then
    print("Toast failed: " .. tostring(err))
    os.exit()
  end
end

-- Try to get ActivityManager safely
local am
local ok, err = pcall(function()
  am = ctx:getSystemService(Context.ACTIVITY_SERVICE)
end)

if not ok or not am then
  showToastAndExit("Failed to access ActivityManager: " .. tostring(err))
  return
end

-- Try to kill background apps safely
local success, errorMessage = pcall(function()
  local running = am:getRunningAppProcesses()
  if not running then
    error("No running apps found.")
  end

  local count = 0
  for i = 0, running:size() - 1 do
    local proc = running:get(i)
    local name = proc.processName
    if name and name ~= ctx:getPackageName() then
      local killed, killErr = pcall(function()
        am:killBackgroundProcesses(name)
        count = count + 1
      end)
      if not killed then
        print("Failed to kill " .. name .. ": " .. tostring(killErr))
      end
    end
  end

  showToastAndExit("Killed " .. count .. " background apps.")
end)

-- If killing failed
if not success then
  showToastAndExit("Error: " .. tostring(errorMessage))
end